@@ -24,7 +24,7 @@ from fabrics.components.leaves.leaf import Leaf
 from fabrics.components.leaves.attractor import GenericAttractor
 from fabrics.components.leaves.dynamic_attractor import GenericDynamicAttractor
 from fabrics.components.leaves.dynamic_geometry import DynamicObstacleLeaf, GenericDynamicGeometryLeaf
-from fabrics.components.leaves.geometry import ObstacleLeaf, LimitLeaf, SelfCollisionLeaf, GenericGeometryLeaf
+from fabrics.components.leaves.geometry import ObstacleLeaf, LimitLeaf, SelfCollisionLeaf, GenericGeometryLeaf, ESDFGeometryLeaf
 
 from mpscenes.goals.goal_composition import GoalComposition
 from mpscenes.goals.sub_goal import SubGoal
@@ -285,17 +285,41 @@ class ParameterizedFabricPlanner(object):
             )
         return fk
 
+    def get_forward_kinematics_F(self, link_name) -> ca.SX:
+        if isinstance(link_name, ca.SX):
+            return link_name
+        if self._config.urdf:
+            fk = self._forward_kinematics.fk(
+                self._variables.position_variable(),
+                self._config.root_link,
+                link_name,
+                positionOnly=True
+            )
+        else:
+            fk = self._forward_kinematics.fk(
+                self._variables.position_variable(),
+                link_name,
+                positionOnly=True
+            )
+        return fk_fun
+
     """ DEFAULT COMPOSITION """
     def set_components(
         self,
-        collision_links: list,
-        self_collision_pairs: dict,
+        collision_links: list = None,
+        self_collision_pairs: dict = None,
+        collision_links_esdf: list = None,
         goal: GoalComposition = None,
         limits: list = None,
         number_obstacles: int = 1,
         number_dynamic_obstacles: int = 0,
     ):
-        # Adds default obstacle avoidance
+        if collision_links is None:
+            collision_links = []
+        if collision_links_esdf is None:
+            collision_links_esdf = []
+        if self_collision_pairs is None:
+            self_collision_pairs = {}
         reference_parameter_list = []
         for i in range(number_dynamic_obstacles):
             reference_parameters = {
@@ -321,6 +345,15 @@ class ParameterizedFabricPlanner(object):
                 geometry.set_geometry(self.config.collision_geometry)
                 geometry.set_finsler_structure(self.config.collision_finsler)
                 self.add_leaf(geometry)
+
+
+        for collision_link in collision_links_esdf:
+            fk = self.get_forward_kinematics(collision_link)
+            geometry = ESDFGeometryLeaf(self._variables, collision_link, fk)
+            geometry.set_geometry(self.config.collision_geometry)
+            geometry.set_finsler_structure(self.config.collision_finsler)
+            self.add_leaf(geometry)
+
         for self_collision_key, self_collision_list in self_collision_pairs.items():
             fk_key = self.get_forward_kinematics(self_collision_key)
             for self_collision_link in self_collision_list:
@@ -397,7 +430,11 @@ class ParameterizedFabricPlanner(object):
             self.add_leaf(attractor, prime_leaf=sub_goal.is_primary_goal())
 
 
-    def concretize(self):
+    def concretize(self, mode='acc', time_step=None):
+        self._mode = mode
+        if mode == 'vel':
+            if not time_step:
+                raise Exception("No time step passed in velocity mode.")
         try:
             eta = self._damper.substitute_eta()
             a_ex = (
@@ -416,9 +453,16 @@ class ParameterizedFabricPlanner(object):
             #xddot = self._geometry._xddot - self._geometry._alpha * self._geometry._vars.velocity_variable()
             xddot = self._execution_geometry._xddot - self._execution_geometry._alpha * self._geometry._vars.velocity_variable()
 
-        self._funs = CasadiFunctionWrapper(
-            "funs", self.variables.asDict(), {"xddot": xddot}
-        )
+        if mode == 'acc':
+            self._funs = CasadiFunctionWrapper(
+                "funs", self.variables.asDict(), {"action": xddot}
+            )
+        elif mode == 'vel':
+            action = self._geometry.xdot() + time_step * xddot
+            self._funs = CasadiFunctionWrapper(
+                "funs", self.variables.asDict(), {"action": action}
+            )
+            
 
     def serialize(self, file_name: str):
         """
@@ -441,7 +485,7 @@ class ParameterizedFabricPlanner(object):
         The action is nullified if its magnitude is very large or very small.
         """
         evaluations = self._funs.evaluate(**kwargs)
-        action = evaluations["xddot"]
+        action = evaluations["action"]
         # Debugging
         #logging.debug(f"a_ex: {evaluations['a_ex']}")
         #logging.debug(f"alhpa_forced_geometry: {evaluations['alpha_forced_geometry']}")
@@ -456,10 +500,4 @@ class ParameterizedFabricPlanner(object):
             action *= 0.0
         return action
 
-    """
-    def __del__(self):
-        del(self._variables)
-        print("PLANNER DELETED")
-    """
-
 
